#!/usr/bin/perl
#
# ib-convert.pl -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

use strict;
use warnings;

my $file = shift;
unless ( defined($file) ) {
  print "ib-convert.pl (c) Oliver Bossert 2004\n";
  print "For more information and licence: http://www.olfsworld.de/software/otj/\n\n";
  print "usage: $0 [options] <filename>\n";
  exit;
}

#
# Extract the data
#
print "# This data was extracted by ib-convert (c) Oliver Bossert 2005\n";
print "# You can import the data in OTJ (Olfs Trading Journal).\n";
print "# This software can be found at: http://www.olfsworld.de/software/otj/\n";
my @positions = ();
open F, $file or die "Could not open file: $file";
while (<F>) {
  chomp;
  s/\#.*$//;
  next if (/^\s*$/);
  push @positions, [ split /;/, $_ ];
}
close F;

# Generate the trades
my %depot = ();
my $id = 0;
print '# ' . join("\t", qw(Id Code Nb direction OpenDate
			   OpenVal CloseDate CloseVal Fee Comment)) . "\n";
foreach my $pos ( sort { $a->[0] cmp $b->[0] } @positions ) {

#  print  "=" x 80 . "\n";
#  print  join("\n", @$pos) . "\n";
#  foreach my $c ( keys %depot ) {
#    print  "  => " . $c . " --> " . $depot{$c}{nb} . "\n";
#    foreach my $a ( @{$depot{$c}{ob}} ) {
#      print  "        => " . join("; ", @{$a}) . "\n";
#    }
#  }
#  print  "=" x 80 . "\n";

  # No stocks of this type in the depot
  if ( !defined($depot{$pos->[0]}{nb}) || 
       $depot{$pos->[0]}{nb} == 0 ) {
    if ( $pos->[1] =~ /SLD/ ) {
      $depot{$pos->[0]}{nb} = -$pos->[2];
    } else {
      $depot{$pos->[0]}{nb} = $pos->[2];
    }
    push @{$depot{$pos->[0]}{ob}}, $pos;
  }

  # There are some stocks in the depot ...
  else {

    # Add the stocks
    if ( $pos->[1] =~ /SLD/ &&
	 defined($depot{$pos->[0]}{nb}) &&
	 $depot{$pos->[0]}{nb} < 0 ) {
      $depot{$pos->[0]}{nb} -= $pos->[2];
      push @{$depot{$pos->[0]}{ob}}, $pos;
    }
    elsif ( $pos->[1] =~ /BOT/ &&
	    defined($depot{$pos->[0]}{nb}) &&
	    $depot{$pos->[0]}{nb} > 0 ) {
      $depot{$pos->[0]}{nb} += $pos->[2];
      push @{$depot{$pos->[0]}{ob}}, $pos;
    }

    # Buy / Sell for 0,00
#    elsif ( $pos->[1] =~ /^ausgebucht/ ) {
#    }

    # Generate a trade
    else {

      # Go through the trades
      my $nb= $pos->[2];

      if ( $pos->[1] =~ /SLD/ ) {

	$depot{$pos->[0]}{nb} -= $pos->[2];

	foreach my $trade ( sort {$a->[4] cmp $b->[4]} @{$depot{$pos->[0]}{ob}} ) {
	  last if ( $nb == 0 );
	  next if ( $trade->[2] == 0 );

	  if ( $trade->[1] =~ /BOT/ ) {
	    my $tmp_nb;
	    if ( $trade->[2] >= $nb ) {
	      $tmp_nb = $nb;
	      $trade->[2] -= $nb;
	      $nb = 0;
	    } else {
	      $tmp_nb = $trade->[2];
	      $nb -= $trade->[2];
	      $trade->[2] = 0;
	    }

	    print sprintf("%06d", $id++) . "\t" . $pos->[0] . "\t" . $tmp_nb;
	    print "\tlong";
	    print "\t" . join("\t", ( join("-", ($trade->[5] =~ /(\d{4})(\d{2})(\d{2})/ )) . 
				      "_" . $trade->[4], # Open Date
				      $trade->[3], # Open Price
				      join("-", ($pos->[5] =~ /(\d{4})(\d{2})(\d{2})/ )) . 
				      "_" . $pos->[4],
				      $pos->[3],
				      sprintf("%.2f", (0.003*$tmp_nb*$trade->[3] + 0.003*$tmp_nb*$pos->[3])),
				      $pos->[6] . "/" . $pos->[7]
				    )) . "\n";

	  }
	}
      }

      # Close a short-position
      else {

	$depot{$pos->[0]}{nb} += $pos->[2];

	foreach my $trade ( sort {$a->[4] cmp $b->[4]} @{$depot{$pos->[0]}{ob}} ) {
	  last if ( $nb == 0 );
	  next if ( $trade->[2] == 0 );

	  if ( $trade->[1] =~ /SLD/ ) {

	    my $tmp_nb;
	    if ( $trade->[2] >= $nb ) {
	      $tmp_nb = $nb;
	      $trade->[2] -= $nb;
	      $nb = 0;
	    } else {
	      $tmp_nb = $trade->[2];
	      $nb -= $trade->[2];
	      $trade->[2] = 0;
	    }

	    print sprintf("%06d", $id++) . "\t" . $pos->[0] . "\t" . $tmp_nb;
	    print "\tshort";
	    print "\t" . join("\t", ( join("-", ($trade->[5] =~ /(\d{4})(\d{2})(\d{2})/ )) . 
				      "_" . $trade->[4], # Open Date
				      $trade->[3], # Open Price
				      join("-", ($pos->[5] =~ /(\d{4})(\d{2})(\d{2})/ )) . 
				      "_" . $pos->[4],
				      $pos->[3],
				      sprintf("%.2f", (0.003*$tmp_nb*$trade->[3] + 0.003*$tmp_nb*$pos->[3])),
				      $pos->[6] . "/" . $pos->[7]
				    )) . "\n";

	  }
	}
      }

      $pos->[2] = $nb;
      push @{$depot{$pos->[0]}{ob}}, $pos;

#      print STDERR ">>>>>" . $pos->[0] . " ==> " . $pos->[2] . " ($nb)\n";

    }
  }
}
